global target

set crc_poly 0x8005

proc crc16 {fd size poly} {
	set n $size
	set crc 0
	set data 0

	while { $n > 0} {
		set data [read $fd 1]
		binary scan $data c1 data
		set data [expr $data&0xFF]

		for { set i 0} { $i < 8 } { incr i } {
			set d [expr ($data>>$i)&1]
			set c [expr $crc>>15]
			set c [expr $c&0xFFFF]
			set crc [expr $crc<<1]
			set crc [expr $crc&0xFFFF]
			if {$d != $c} {
				set crc [expr $crc^$poly]
			}
		}
		set n [expr $n-1]
	}
	return $crc
}


set table_addr 0x20000
set gif_addr 0x20100
set err_code 0
set gif_size_list {}
set gif_addr_list {}
set gif_crc_list {}
set nvm_page_size 64
set nvm_row_size [expr $nvm_page_size*4]

set gif_files [list \
"r:/GIFs/running_dog.gif" \
"r:/GIFs/bg_grau13.gif" \
"r:/GIFs/ds-digi.gif"]

set gif_num [llength $gif_files]

for { set i 0} { $i < $gif_num } { incr i } {
	set gif_fsize [file size [lindex $gif_files $i]]
	lappend gif_size_list $gif_fsize
	lappend gif_addr_list $gif_addr

	set gif_fd [open [lindex $gif_files $i] r]
	fconfigure $gif_fd -translation binary
	set gif_crc [crc16 $gif_fd $gif_fsize $crc_poly]
	close $gif_fd
	lappend gif_crc_list $gif_crc

	set gif_addr [expr $gif_addr+(($gif_fsize/$nvm_row_size)+1)*$nvm_row_size]
}

puts $gif_size_list
puts $gif_addr_list

set table_fname "r:/tmp/gif_table.tbl"
set table_fd [open $table_fname w]
fconfigure $table_fd -translation binary

# Compose the GIF descriptor table
set to_table ""
for { set i 0} { $i < $gif_num } { incr i } {
	puts stdout "Size of $i. GIF: [lindex $gif_size_list $i]"
	append to_table [binary format ssii 0x0 [lindex $gif_crc_list $i] [lindex $gif_size_list $i] [lindex $gif_addr_list $i]]
}

# Erase the flash memory, where the table will be located
for { set i 0} { $i < [string length $to_table] } { incr i } {
	puts -nonewline $table_fd [binary format S 0xFFFF]
}
flush $table_fd
DATAFLASH::sendFile $table_fname $table_addr
seek $table_fd 0

# Send the GIF descriptor table to flash
puts -nonewline $table_fd $to_table
flush $table_fd
DATAFLASH::sendFile $table_fname $table_addr

proc send_gif { fname addr } {
	set tmp_fname "r:/tmp/tmp.bin"
	set gif_fsize [file size $fname]

	if {[expr $gif_fsize % 2] != 0} {
		# SAMBA bug: odd file length causes bad last character written
		# -> extend file by 0xFF
		set gif_fd [open $fname r]
		fconfigure $gif_fd -translation binary
		set data [read $gif_fd]
		close $gif_fd

		set fname $tmp_fname

		set gif_fd [open $tmp_fname w]
		fconfigure $gif_fd -translation binary
		puts -nonewline $gif_fd $data
		puts -nonewline $gif_fd [binary format c1 0xFF]
		close $gif_fd
	}

	DATAFLASH::sendFile $fname $addr
}

# Send all the GIFs to flash
for { set i 0} { $i < $gif_num } { incr i } {
	send_gif [lindex $gif_files $i] [lindex $gif_addr_list $i]
}

close $table_fd
